/*
 * QuteCom, a voice over Internet phone
 * Copyright (C) 2010 Mbdsys
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef OWDATE_H
#define OWDATE_H

#include <util/owutildll.h>

#include <ctime>
#include <string>

/**
 * Represents a date.
 *
 * @author Philippe Bernery
 */
class OWUTIL_API Date {
	friend class DateXMLSerializer;
public:

	/**
	 * Creates a date with the current time.
	 */
	 Date();

	/**
	 * Copy constructor.
	 */
	 Date(const Date & date);

	/**
	 * Creates a date from a day a month and a year.
	 *
	 * @param day day of the date
	 * @param month month of the date
	 * @param year year of the date
	 */
	 Date(unsigned day, unsigned month, unsigned year);

	 ~Date();

	 bool operator==(const Date & date) const;

	/**
	 * @return the day number of the date. Day range: 1-31.
	 */
	 unsigned getDay() const;

	/**
	 * Sets the date day.
	 *
	 * @see getDay()
	 */
	 void setDay(unsigned day);

	/**
	 * @return the month number of the date. Month range: 1-12.
	 */
	 unsigned getMonth() const;

	/**
	 * Sets the date month.
	 *
	 * @see getMonth()
	 */
	 void setMonth(unsigned month);

	/**
	 * Gets the date year. Year range: 0-infinite.
	 */
	 unsigned getYear() const;

	/**
	 * Sets date year.
	 *
	 * @see getYear()
	 */
	 void setYear(unsigned year);

	/**
	 * @return a string representing the date. (e.g: "yyyy-mm-dd")
	 */
	 std::string toString() const;

private:
	unsigned _day;
	unsigned _month;
	unsigned _year;
};

#endif	//OWDATE_H
